/******************** (C) COPYRIGHT 2005 STMicroelectronics ********************
* File Name          : 91x_mac.h
* Author             : MCD Application Team
* Date First Issued  : 21/10/2005 :  V1.0
* Description        : This file contains all the functions prototypes for the
*                      ENET Ethernet software library.
**********************************************************************************
* History:
* 21/10/2005 :  V1.0
**********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
* CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
* OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
* OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
* CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*********************************************************************************/

/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __91x_ENET_H
#define __91x_ENET_H

/* Includes ------------------------------------------------------------------*/
#include "91x_map.h"

/* Exported types ------------------------------------------------------------*/
typedef enum { READ, WRITE } AccessOperation;
/* WIU Init structure type define */
typedef struct
{
  u32   ENET_GlobalMode ;
  u32   ENET_Prescaler;
  u32   ENET_AddressFilteringMode;
  u32	ENET_BackOfLimit;
  u32	ENET_ReceiveMode;
  u32	ENET_Endianity;
  u32	ENET_WrongFrameFilter;
  u32	ENET_ReverseControlFrameAddress;
  u32	ENET_VCIFrameFiltering;
  FunctionalState	ENET_LateCollision;
  FunctionalState	ENET_BroadCastFrame;
  FunctionalState	ENET_PacketRetry;
  FunctionalState	ENET_AutomaticPadRemoval;
  FunctionalState	ENET_DefferalCheck;
}ENET_InitTypeDef ;

typedef struct  ENET_DMA_DSCRIPTOR {

  u32 DmaStatCntl;           /* DMA Status and Control Register          */
  u32 DmaAddr;               /* DMA Start Address Register               */
  u32 DmaNext;               /* DMA Next Descriptor Register             */
  u32 DmaPackStatus;         /* DMA PAcket Status and Control Register   */

} ENET_DMA_DSCR_Base;

/************************************************************/
/* Ethernet data structure   */
/************************************************************/
typedef struct ENET_Addr
{
   u8 addr[6];

}ENET_addr;

/************************************************************/
/* DMAC ethernet data structure   */
/************************************************************/
typedef struct DmacEthernet {

  ENET_addr ENET_Addr;

  u32 phyAddress;                           /* PHY address of the PHY chip connected to the MAC */

  u32 TxPacketsDropped;                       /* counter for dropped tx packets (e.g. no memory...) */

  ENET_DMA_DSCR_Base * volatile DmaTxDscrBase;     /* base address of DMA TX Descriptors Chain */
  ENET_DMA_DSCR_Base * volatile DmaRxDscrBase;     /* base address of DMA RX Descriptors Chain */

}DmacEthernetDataStruct;

/* Exported constants --------------------------------------------------------*/
/* Global mode */
#define ENET_FullDuplexMode             0x0
#define ENET_HalfDuplexMode	       0x1

/* ENET prescaler */
#define ENET_PS50		       0x2
#define ENET_PS50_100	               0x3
#define ENET_PS100_150	               0x4
#define ENET_PS150_200	               0x5
/* Address filterinf modes */
#define ENET_PhyMultiCastPerfectMode    0x6
#define ENET_PhyPerfectMultiCastHashMode	  0x7
#define ENET_PhyMultiCastHashMode	      0x8
#define ENET_InvMode					      0x9
#define ENET_PromiscousMode	              0x10
#define ENET_PhyPerfectMode	              0x11
#define ENET_PhyHashMode	                  0x12
/* Back off Limit */
#define ENET_8SlotTimes   	               0x13
#define ENET_6SlotTimes	                       0x14
#define ENET_3SlotTimes  	               0x15
#define ENET_10SlotTimes                        0x26
/* Receive mode */
#define ENET_AFMReceiveMode	           0x16
#define ENET_ReceiveAllMode	           0x17
/* Endianity */
#define ENET_LittleEndian	           0x18
#define ENET_BigEndian	               0x19
/* Wrong frame filter */
#define ENET_WrongFrameRejected	          0x20
#define ENET_WrongFramePassed	          0x21

/* VCI Frame filtering */
#define VCI_AllFrameReception				0x22
#define VCI_FilterFailedFrame				0x23

/* ReverseControlFrameAddress */
#define ENET_MCRightToLeft					  0x24
#define ENET_MCRLeftToRight					  0x25
/* Deffereal check */
/* ENET clocks mask */
#define ENET_Simple 0x27
#define ENET_Double 0x28

/* ENET Flags */
#define ENET_FLAG_TFA	0x2800
#define ENET_FLG_TNC	0x2801
#define ENET_FLAG_TLOC	0x2802
#define ENET_FLAG_TED	0x2803
#define ENET_FLAG_TLC	0x2804
#define ENET_FLAG_TEC	0x2805
#define ENET_FLAG_TUR	0x2806
#define ENET_FLAG_TDEF	0x2807
#define ENET_FLAG_TLCO	0x2808
#define ENET_FLAG_TPR	0x2831
#define ENET_FLAG_RFCI	0x2C13
#define ENET_FLAG_RWT	0x2C14
#define ENET_FLAG_RRF	0x2C15
#define ENET_FLAG_ROL	0x2C16
#define ENET_FLAG_RLC	0x2C17
#define ENET_FLAG_RFT	0x2C18
#define	ENET_FLAG_RME	0x2C19
#define ENET_FLAG_REB	0x2C20
#define ENET_FLAG_RCE	0x2C21
#define ENET_FLAG_VL1	0x2C22
#define ENET_FLAG_VL2	0x2C23
#define ENET_FLAG_RLE	0x2C24
#define ENET_FLAG_RCF	0x2C25
#define ENET_FLAG_RUCF	0x2C26
#define ENET_FLAG_RMCF	0x2C27
#define ENET_FLAG_RBF	0x2C28
#define ENET_FLAG_RFF	0x2C29
#define ENET_FLAG_RPF	0x2C30
#define ENET_FLAG_RFA	0x2C31


/************************************************************/
/* MAC ETHERNET ADDRESS */
/************************************************************/
#define DMAC0_ETHADDR0 0x0C
#define DMAC0_ETHADDR1 0x0A
#define DMAC0_ETHADDR2 0x08
#define DMAC0_ETHADDR3 0x04
#define DMAC0_ETHADDR4 0x02
#define DMAC0_ETHADDR5 0x01


/* Exported constants --------------------------------------------------------*/
/* Module private variables --------------------------------------------------*/
/* Exported ENETro ------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */

void ENET_Init(ENET_InitTypeDef* ENET_InitStruct);
void ENET_DeInit(void);
void ENET_StructInit(ENET_InitTypeDef* ENET_InitStruct);
void ENET_LoopBackCmd(FunctionalState NewState);
void ENET_SetMACAddr(void);
void ENET_SetMulticastAddress(u32 wHAddress, u32 wLAddress);
u64 ENET_GetMACAddr(void);
u32 ENET_GetMulticastAddr(void);
u8 ENET_GetPhyAddress(void);
u8 ENET_GetMIIAddr(void);
void ENET_PhyOperationConfig(AccessOperation wOperation);
void ENET_PhyBusyConfig(FunctionalState NewState);
void ENET_ResetMIIRegValues(void);
u16 ENET_MII_Read(u32 PhyBaseAddr , u32 PhyReg);
void ENET_MII_Write(u32 PhyBaseAddr, u32 PhyReg, u16 Data);
void ENET_SetControlFramePauseTime(u16 hPtime);
void ENET_EnableControlFlow(void);
void ENET_FlowControlBusyConfig(void);
u32 ENET_GetVLANIdentifier(void);
u32 ENET_GetVLANTag(void);
u8 ENET_GetCollisionCounter(void);
u32 ENET_GetByteCounter(void);
void ENET_ClockConfig(u8 bClockFactor);
FlagStatus ENET_GetFlagStatus(u16 ENET_Flag);
void DENET_HwReset(void);
void RxDMACDscrInit(DmacEthernetDataStruct* pDMACIf);
DmacEthernetDataStruct* GetDMACContext(void);
u32 DMACHandleRxPkt ( void* ppkt, u16   size);
void TxDMACDscrInit( DmacEthernetDataStruct *ptDMACIf);

u32 DMACRxPacketGetSize(void);
u32 DMACStart(DmacEthernetDataStruct *ptDMACIf);
void LowLevelInit(DmacEthernetDataStruct  *ptDMACIf);
#endif /* __91x_ENET_H */

/******************* (C) COPYRIGHT 2005 STMicroelectronics *****END OF FILE****/

