/******************** (C) COPYRIGHT 2005 STMicroelectronics ********************
* File Name          : 91x_enet.c
* Author             : MCD Application Team
* Date First Issued  : 24/10/2005 :  V1.0
* Description        : This file provides all the ENET Ethernet software functions.
**********************************************************************************
* History:
* 24/10/2005 :  V1.0
**********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
* CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
* OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
* OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
* CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*********************************************************************************/

/* Standard include ----------------------------------------------------------*/
#include "91x_enet.h"
#include "91x_denet.h"
#include "91x_scu.h"
//#include "lib_mem.h"
//#include "lib_mem.h"
//#include "rakmem.h"


/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*Global Modes Masks */
#define FullDuplexMask 0x00200000
#define HalfDuplexMask 0xFFDFFFFF


/* ENET Prescalers */
#define Ps50Mask      0xF9FFFFFF
#define Ps50_100Mask  0xFBFFFFFF
#define Ps100_150Mask 0xFDFFFFFF
#define Ps150_200Mask 0x05000000

/* Address Filtering Mode masks */
#define MCPerfectModeMask     0xFFF1FFFF
#define MCPerfectHashModeMask 0x00020000
#define MCHashModeMask        0x00040000
#define InvModeMask           0x00050000
#define PromiscousModeMask    0x00080000
#define PhyPerfectModeMask    0x000A0000
#define PhyHashModeMask     0x000C0000

/* Back of limit values */
#define ESlotMask  0xFFFFFF7F
#define SSlotMask  0xFFFFFFBF
#define ThSlotMask 0x000000C0
#define TSlotMask  0xFFFFFF3F
/*Receive mode masks */
#define AFMRM_Mask      0x7FFFFFFF
#define ReceiveAll_Mask 0x80000000

/*Endianity modes masks */
#define LittleEndianMask 0xCFFFFFFF
#define BigEndianMask    0x30000000

/*Wrong Frame Filter config Masks */
#define RejectMask   0xFFFEFFFF
#define NoRejectMask 0x00010000

/*Late Collision Config Masks*/
#define LCDisableMask  0xFFFFEFFF
#define LCEnableMask   0x00001000

/*VCI Frame filtering Selection Masks */
#define AllFrameMask    0xFFFFFDFF
#define FilterframeMask 0x00000200

/* BroadCast frame config Masks */
#define BCFEnable  0xFFFFF7FF
#define BCFDisable 0x00000800

/* Packet Retry Config Masks */
#define PREnable    0xFFFFFBFF
#define PRDisable   0x00000400

/* Automatic Pad Removal config Masks */
#define APREnable   0x00000100
#define APRDisable  0xFFFFFEFF

/* Deffreal check  config Masks */
#define DCEnable  0x00000040
#define DCDisable 0xFFFFFFBF

/* Reverse Control Frame address Masks */
#define MCRightToLeftMask  0x00000001
#define MCLeftToRightMask  0xFFFFFFFE

/* Internal LoopBack Masks */
#define LoopBackMask   0xFFBFFFFF
#define NoLoopBackMask 0xFF9FFFFF

/*Read - Write operation mask */
#define ReadMask  0xFFFFFFFD
#define WriteMask 0x00000002

/* FrameControl Masks */
#define FCEnableMask  0x00000002
#define FCDisableMask 0xFFFFFFFD

/* VLAN Masks */
#define VLID_Mask 0x0FFF0000;
#define VLTAG_Mask 0x0000FFFF;

/* ENET Satus Mask */
#define CC_Mask 0x00003C00;
#define BC_Mask 0x7FCC0000;

/*Clock relation Mask */
#define SimpleClock_Mask 0xFFFFFFF3
#define DoubleClock_Mask 0x00000004

/*MTS & MRS access address */
#define MTS_Addr 0x2800;
#define MRS_Addr 0x2C00;

/* Private macrro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
 u8 tempRxAry[1536];
 u8 tempTxAry[1536];
 static  DmacEthernetDataStruct                *gPTDmacif;
/* Private function prototypes -----------------------------------------------*/
/* Interface functions -------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/******************************************************************************
* Function Name  : ENET_Init
* Description    : Initializes the ENET unit according to the specified parameters
*                  in the ENET_InitTypeDef structure.
* Input          : ENET_InitStruct: pointer to a ENET_InitTypeDef structure that
*                  contains the configuration information for the ENET peripheral.
* Output         : None
* Return         : None
******************************************************************************/
void ENET_Init(ENET_InitTypeDef* ENET_InitStruct)
{
 /* Global Mode Selection */
 if (ENET_InitStruct->ENET_GlobalMode == ENET_FullDuplexMode)
  {
   ENET->MCR |= FullDuplexMask;
  }

 if (ENET_InitStruct->ENET_GlobalMode == ENET_HalfDuplexMode)
  {
   ENET->MCR &= HalfDuplexMask;
  }



  /* Set ENET Prescaler  */
 if (ENET_InitStruct ->ENET_Prescaler == ENET_PS50)
  {
   ENET->MCR &= Ps50Mask;
  }

 if (ENET_InitStruct ->ENET_Prescaler == ENET_PS50_100)
  {
   ENET->MCR &= Ps50_100Mask;
  }

 if (ENET_InitStruct ->ENET_Prescaler == ENET_PS100_150)
  {
   ENET->MCR &= Ps100_150Mask;
  }

 if (ENET_InitStruct ->ENET_Prescaler == ENET_PS150_200)
  {
   ENET->MCR |= Ps150_200Mask;
  }


 /* Set Address Filtering Mode */
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_PhyMultiCastPerfectMode)
  {
   ENET->MCR &=MCPerfectModeMask;
  }
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_PhyPerfectMultiCastHashMode)
  {
   ENET ->MCR |=MCPerfectHashModeMask;
  }
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_PhyMultiCastHashMode)
  {
   ENET ->MCR|=MCHashModeMask;
  }
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_InvMode)
  {
   ENET ->MCR |=InvModeMask;
  }
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_PromiscousMode)
  {
   ENET ->MCR |=PromiscousModeMask;
  }
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_PhyPerfectMode)
  {
   ENET ->MCR |=PhyPerfectModeMask;
  }
 if (ENET_InitStruct ->ENET_AddressFilteringMode == ENET_PhyHashMode)
  {
   ENET ->MCR |=MCPerfectModeMask;
  }

 /* Set Back of limit */
 if (ENET_InitStruct ->ENET_BackOfLimit== ENET_8SlotTimes)
  {
   ENET ->MCR &=ESlotMask; /* Eight Slot times */
  }
 if (ENET_InitStruct ->ENET_BackOfLimit== ENET_6SlotTimes)
  {
   ENET ->MCR &=SSlotMask; /* Six Slot times */
  }
 if (ENET_InitStruct ->ENET_BackOfLimit== ENET_3SlotTimes)
  {
   ENET ->MCR |=ThSlotMask; /* Three Slot times */
  }
 if (ENET_InitStruct ->ENET_BackOfLimit== ENET_10SlotTimes)
  {
   ENET ->MCR &=TSlotMask; /* Ten Slot times */
  }
 /*Set receive Mode */
 if (ENET_InitStruct ->ENET_ReceiveMode== ENET_AFMReceiveMode)
  {
   ENET ->MCR &=AFMRM_Mask;
  }
  else
   {
    ENET ->MCR|=ReceiveAll_Mask;
   }

/*Set Endianity mode */
 if (ENET_InitStruct ->ENET_Endianity== ENET_LittleEndian)
  {
   ENET ->MCR &=LittleEndianMask;
  }
  else
   {
    ENET ->MCR|=BigEndianMask;
   }

 /* Set Wrong frame filter  */
 if (ENET_InitStruct ->ENET_WrongFrameFilter== ENET_WrongFrameRejected)
  {
   ENET ->MCR &= RejectMask;
  }
  else
   {
    ENET ->MCR|= NoRejectMask;
   }

 /* Set Late collision */
 if (ENET_InitStruct ->ENET_LateCollision== ENABLE)
  {
   ENET->MCR|=LCEnableMask;
  }
  else
   {
    ENET->MCR &=LCDisableMask;
   }


  /* VCI Frame filtering Selection */
 if (ENET_InitStruct ->ENET_VCIFrameFiltering== VCI_AllFrameReception)
  {
   ENET->MCR &=AllFrameMask;
  }
  else
   {
    ENET->MCR |=FilterframeMask;
   }

  /* BroadCast frame config */

 if (ENET_InitStruct ->ENET_BroadCastFrame== ENABLE)
  {
   ENET->MCR&=BCFEnable;
  }
  else
   {
    ENET->MCR|=BCFDisable;
   }

  /* Packet Retry Config */

 if (ENET_InitStruct ->ENET_PacketRetry==ENABLE)
  {
   ENET->MCR&= PREnable;
  }
  else
   {
    ENET->MCR|= PRDisable;
   }


  /* Automatic Pad Removal config */

 if (ENET_InitStruct ->ENET_AutomaticPadRemoval==ENABLE)
  {
   ENET->MCR|=APREnable;
  }
  else
   {
    ENET->MCR&=APRDisable;
   }

  /* Deffreal check  config */

 if (ENET_InitStruct->ENET_DefferalCheck==ENABLE)
  {
   ENET->MCR|=DCEnable;
  }
  else
   {
    ENET->MCR&=DCDisable;
   }


  /* Reverse Control Frame address */

 if (ENET_InitStruct ->ENET_ReverseControlFrameAddress==ENET_MCRightToLeft)
  {
   ENET->MCR |= MCRightToLeftMask;
  }
  else
   {
    ENET->MCR&= MCLeftToRightMask;
   }
   ENET->MCR|=0xC;
}


/******************************************************************************
* Function Name  : ENET_DeInit
* Description    : Deinitializes the ENET cell
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/

void ENET_DeInit(void)
{

  SCU_AHBPeriphReset(__MAC, ENABLE);
  SCU_AHBPeriphReset(__MAC, DISABLE);

}

/*******************************************************************************
* Function Name  : ENET_StructInit		         		
* Description    : Initialize the ENET Init Structure parameters
* Input          : - ENET_InitStruct: pointer to a ENET_InitTypeDef structure
                     which will be initialized.
* Output         : None	
* Return         : None.						
*******************************************************************************/
void ENET_StructInit(ENET_InitTypeDef* ENET_InitStruct)
{
  /* Initialize the ENET_Global mode member */
  ENET_InitStruct->ENET_GlobalMode = ENET_HalfDuplexMode;

  /* Initialize the ENET_Prescaler member */
  ENET_InitStruct->ENET_Prescaler = ENET_PS50;

  /* Initialize the ENET_AddressFilteringMode member */
  ENET_InitStruct->ENET_AddressFilteringMode = ENET_PhyMultiCastPerfectMode;

  /* Initialize the ENET_BackOfLimit member */
  ENET_InitStruct->ENET_BackOfLimit = ENET_10SlotTimes;

  /* Initialize the ENET_ReceiveMode member */
  ENET_InitStruct->ENET_ReceiveMode= ENET_AFMReceiveMode;

  /* Initialize the ENET_Endianity member */
  ENET_InitStruct->ENET_Endianity= ENET_LittleEndian;

  /* Initialize the ENET_WrongFrameFilter member */
  ENET_InitStruct->ENET_WrongFrameFilter= ENET_WrongFrameRejected;

  /* Initialize the ENET_LateCollision member */
  ENET_InitStruct->ENET_LateCollision= DISABLE;

  /* Initialize the ENET_VCIFrameFiltering member */
  ENET_InitStruct->ENET_VCIFrameFiltering= VCI_AllFrameReception;

  /* Initialize the ENET_BroadCastFrame member */
  ENET_InitStruct->ENET_BroadCastFrame= ENABLE;

  /* Initialize the ENET_PacketRetry member */
  ENET_InitStruct->ENET_PacketRetry= ENABLE;

  /* Initialize the ENET_AutomaticPadRemoval member */
  ENET_InitStruct->ENET_AutomaticPadRemoval=DISABLE;

  /* Initialize the ENET_DefferalCheck member */
  ENET_InitStruct->ENET_DefferalCheck= DISABLE;

  /* Initialize the ENET_ReverseControlFrameAddress member */
  ENET_InitStruct->ENET_ReverseControlFrameAddress= ENET_MCRLeftToRight;
}


/*******************************************************************************
* Function Name  : ENET_LoopBackCmd		         		
* Description    : Enable or diable the ENET internal loppback
* Input          : - NewState: The new state of the internal loopback
                               enabled or disabled.
* Output         : None	
* Return         : None.						
*******************************************************************************/


void ENET_LoopBackCmd(FunctionalState NewState)
{
 if (NewState == ENABLE)
  {
   ENET->MCR&=LoopBackMask;
  }
  else
   {
    ENET->MCR&=NoLoopBackMask;
   }
}

/*******************************************************************************
* Function Name  :ENET_SetMacAddr		         		
* Description    :Sets the MAC Address which will be an identifier of the ENET
                  controller in the network.
* Input          :- hAddrHigh: The High part of the ENET address it will be
                                setted on the MAHR register.
                  - wAddrLow:  The Low part of the ENET address it will be
                                setted on the MALR register.
* Output         : None	
* Return         : None.						
*******************************************************************************/
void ENET_SetMACAddr(void)
{
  u32 macAddrLow;
  u32 macAddrHigh;

	
  macAddrLow  = (DMAC0_ETHADDR3<<24) + (DMAC0_ETHADDR2<<16) + \
                (DMAC0_ETHADDR1<<8) + DMAC0_ETHADDR0;
  macAddrHigh = (DMAC0_ETHADDR5<<8) + DMAC0_ETHADDR4;

  /* write the Mac address Low Register */
  ENET->MALR&=macAddrLow;
  ENET->MAHR&=macAddrHigh;

}

/*******************************************************************************
* Function Name  :RxDMACDscrInit		         		
* Description    :initialize the Rx Descriptor
* Input          :- ptDMACIf: a pointer to DmacEthernetDataStruct struct
* Output         : None	
* Return         : None.						
*******************************************************************************/
void RxDMACDscrInit( DmacEthernetDataStruct *ptDMACIf)
{
	ENET_DMA_DSCR_Base 	*pRXDescrChain;
	

      		
        pRXDescrChain = (ptDMACIf->DmaRxDscrBase);	

        pRXDescrChain->DmaAddr = (u32)tempRxAry;		
        pRXDescrChain->DmaStatCntl = DENET->DSCR;
        pRXDescrChain->DmaNext = (u32)pRXDescrChain;

	/* Setting the RX NEXT Descriptor Register inside the DMA */
        DENET->RxDNDAR = (u32)(ptDMACIf->DmaRxDscrBase); /* mDMA_SetRxNextDescriptor( ... NextDescrAddr); */
   	
	
	/* Setting the VALID bit */
	pRXDescrChain->DmaPackStatus = 0x00010000;

	DENET->RxDBAR |= 0x00000003;	
}

/*******************************************************************************
* Function Name  :TxDMACDscrInit		         		
* Description    :initialize the Tx Descriptor
* Input          :- ptDMACIf: a pointer to DmacEthernetDataStruct struct
* Output         : None	
* Return         : None.						
*******************************************************************************/
void TxDMACDscrInit( DmacEthernetDataStruct *ptDMACIf)
{
	ENET_DMA_DSCR_Base 	* pTXDescrChain;
	

      		
        pTXDescrChain = ptDMACIf->DmaTxDscrBase;	

        pTXDescrChain->DmaAddr = (u32)tempTxAry;		
        pTXDescrChain->DmaStatCntl = DENET->DSCR;
        pTXDescrChain->DmaNext = (u32)pTXDescrChain;

	/* Setting the RX NEXT Descriptor Register inside the DMA */
	DENET->TxDNDAR = (u32)ptDMACIf->DmaTxDscrBase; /* mDMA_SetRxNextDescriptor( ... NextDescrAddr); */
   	
	
	/* Setting the VALID bit */
	pTXDescrChain->DmaPackStatus = 0x00010000;

	DENET->TxDBAR |= 0x00000003;	
}

/*******************************************************************************
* Function Name  :DMACRxPacketGet		         		
* Description    :Get Rx Packet
* Input          :- ptDMACIf: a pointer to DmacEthernetDataStruct struct
* Output         : None	
* Return         : None.						
*******************************************************************************/

ENET_DMA_DSCR_Base * DMACRxPacketGet( DmacEthernetDataStruct *ptDMACIf)
{
      u32      value;

      /* check if the descriptor is owned by the DMA or CPU */
      value = (u32) (ptDMACIf->DmaRxDscrBase->DmaPackStatus & 0x00010000);
	
      if (value == 0x00010000)
      {
           return 0;
      }
	
      return (ptDMACIf->DmaRxDscrBase);
}

/*******************************************************************************
* Function Name  :DMACHandleRxPkt		         		
* Description    :Get Rx Packet
* Input          :- ppkt: pointer to the packet to handle
*                 - size : size of the packet
* Output         : None	
* Return         : None.						
*******************************************************************************/
u32 DMACHandleRxPkt (void* ppkt, u16   size)
{

    ENET_DMA_DSCR_Base   *pDescr;
    DmacEthernetDataStruct       *ptDMACIf;
    ptDMACIf = GetDMACContext();

	/* loop on all descriptor owned by the cpu, checking if the current buffer
	     (rxBuffCurrent) descriptor has the VALID bit reset */
	
	pDescr = DMACRxPacketGet ( ptDMACIf);

	if ( (pDescr != 0)  && (pDescr->DmaAddr)  )
       {		 	
              size = ((pDescr->DmaPackStatus & 0x7ff) - 4);

              Mem_Copy((u8*)ppkt, tempRxAry, size);
       }
	else
		return 0;

	 /* give the buffer back to DMAC */
        pDescr->DmaPackStatus = 0x00010000;

       /* Start the receive operation */
     DENET_Cmd(DENET_RX);

	return 1;
	
}


/*******************************************************************************
* Function Name  :DMACRxPacketGetSize		         		
* Description    :Get Rx Packet Size
* Input          : None
* Output         : None	
* Return         : None						
*******************************************************************************/
u32 DMACRxPacketGetSize(void)
{
     DmacEthernetDataStruct          *ptDMACIf;
     ENET_DMA_DSCR_Base  *pDescr;

     ptDMACIf = GetDMACContext();

     if(ptDMACIf != 0)
     {
        pDescr = DMACRxPacketGet ( ptDMACIf);
		
        if(pDescr != 0)
        {
            return ((pDescr->DmaPackStatus & 0x7ff) - 4);
        }
     }
	
      return 0;
	
}


/*******************************************************************************
* Function Name  :GetDMACContext		         		
* Description    :Get DMAC Context
* Input          : None
* Output         : None	
* Return         : None						
*******************************************************************************/

DmacEthernetDataStruct* GetDMACContext()
{
    return  gPTDmacif;
}

/*******************************************************************************
* Function Name  : LowLevelInit		         		
* Description    : Low Level initialisation
* Input          : ptDMACIf : pointer to DmacEthernetDataStruct struct
* Output         : None	
* Return         : None						
*******************************************************************************/

void LowLevelInit(DmacEthernetDataStruct  *ptDMACIf)
{ 	
   ptDMACIf->DmaTxDscrBase = 0x0;
   ptDMACIf->DmaRxDscrBase = 0x0;


  /* make up a MAC address for DMAC network interface. */
  ptDMACIf->ENET_Addr.addr[0] = DMAC0_ETHADDR0;
  ptDMACIf->ENET_Addr.addr[1] = DMAC0_ETHADDR1;
  ptDMACIf->ENET_Addr.addr[2] = DMAC0_ETHADDR2;
  ptDMACIf->ENET_Addr.addr[3] = DMAC0_ETHADDR3;
  ptDMACIf->ENET_Addr.addr[4] = DMAC0_ETHADDR4;
  ptDMACIf->ENET_Addr.addr[5] = DMAC0_ETHADDR5;

  /* Setup pointers to DMA descriptors and DMA data buffers */
  ptDMACIf->DmaTxDscrBase = (ENET_DMA_DSCR_Base *)mem_malloc(sizeof(ENET_DMA_DSCR_Base) );
  ptDMACIf->DmaRxDscrBase = (ENET_DMA_DSCR_Base *)mem_malloc(sizeof(ENET_DMA_DSCR_Base) );

  /* Remove DMA reset */


  /* Prepare Tx DMA Descriptor */
  TxDMACDscrInit(ptDMACIf);

  /* Prepare Rx DMA Descriptor */
  RxDMACDscrInit(ptDMACIf);


   /* Start the DMA */
  DMACStart(ptDMACIf);

}

/*******************************************************************************
* Function Name  : DMACStart		         		
* Description    : Start DMAc
* Input          : ptDMACIf : pointer to DmacEthernetDataStruct struct
* Output         : None	
* Return         : None						
*******************************************************************************/

 u32 DMACStart ( DmacEthernetDataStruct *ptDMACIf)
{

    u32          value;

  /* Data receive operation */
  /* Write the start address of receive descriptors */

  value  = (u32)(ptDMACIf->DmaRxDscrBase->DmaAddr);
  DENET->RxDBAR |= value;

  /* Data transmit operation */
  /* Write start of transmit descriptors */

  value  = (u32) (ptDMACIf->DmaTxDscrBase->DmaAddr);
  DENET->TxDBAR |=  value;

  /* Setup Descriptor Fetch Delay for Receive Block */

  value=DENET->RxDSR;
  value &= ~( DMA_RX_START_DFETCH_DLY_MASK );
  value |= DMA_RX_START_DFETCH_DEFAULT;
  DENET->RxDSR|= value;


  /* Setup Descriptor Fetch Delay for Transmit Block */

  value=DENET->TxDSR;
  value &= ~( DMA_TX_START_DFETCH_DLY_MASK );
  value |= DMA_TX_START_DFETCH_DEFAULT;
  DENET->TxDSR|= value;

  value &= ~( DMA_TX_START_URUN );
  value |= DMA_TX_START_URUN_MASK;
  DENET->TxDSR|= value;


  return (0);

}

/*******************************************************************************
* Function Name  :ENET_SetMulticastAddr		         		
* Description    :Set the multicast address for filetring.
* Input          :- wAddress: The Multicast address, it will be setted on the
                    MCAHR register
* Output         : None.	
* Return         : None.						
*******************************************************************************/
void ENET_SetMulticastAddress(u32 wHAddress, u32 wLAddress)
{
 ENET->MCAHR|=wHAddress;	
 ENET->MCALR|=wLAddress;
}

/*******************************************************************************
* Function Name  : ENET_GetMacAddr		         		
* Description    : Get the MAC Address.
* Input          : None.
* Output         : None.	
* Return         : 8 byte MAC address						
*******************************************************************************/
u64 ENET_GetMacAddr(void)
{
 u32 wtmp=0x0;
 u64 dwResult=0x0;
 dwResult= (ENET->MAHR) ;
 wtmp = ENET->MALR;
 return ((dwResult<<16)&(wtmp));
}

/*******************************************************************************
* Function Name  : ENET_GetMulticastAddr		         		
* Description    : Gets the multicast address depending of the selected filtering
                   mode, if the perfect filtering mode is selected the function
                   will return the multicast address for the ENET Controller.If the
                   Hash filtering mode is selected, the function will return the
                   Hash Table value.
* Input          : None.
* Output         : None.	
* Return         : Multicast or Hash Table values.						
*******************************************************************************/
u32 ENET_GetMulticastAddr(void)
{
 return (ENET->MCAHR);	
}

/*******************************************************************************
* Function Name  : ENET_MII_WaitNotbusy		         		
* Description    : wait for PHY to accomplish the current task.
* Input          : None.
* Output         : None.	
* Return         : None					
*******************************************************************************/
void ENET_MII_WaitNotbusy(void)
{
 u32 timeOut=0x5000;
  while (ENET->MIIR & 0x01)
  {
   if (!timeOut)
   {
    break;
   }
   timeOut--;
  }
}

/*******************************************************************************
* Function Name  : ENET_GetMIIAddr		         		
* Description    : Gets the MII address
* Input          : None.
* Output         : None.	
* Return         : 1 byte address of MII.						
*******************************************************************************/
u8 ENET_GetMIIAddr(void)
{
 u32 wTmp=0x0;
 wTmp=(ENET->MIIR)&0x07A0;
 return (wTmp<<21);
}

/*******************************************************************************
* Function Name  : ENET_PhyOperationConfig		         		
* Description    : Configure the operation that will be executed by the PHY
                   Device (Write or read).
* Input          : wOperation: The operation type it can be one of the following
		   values:
                      - READ :read operation.
                      - WRITE: write operation.
* Output         : None.	
* Return         : 1 byte address of MII.						
*******************************************************************************/
void ENET_PhyOperationConfig(AccessOperation wOperation)
{
 if (wOperation == READ)
 {
  ENET->MIIR&=ReadMask;
 }
 else
 {
  ENET->MIIR|=WriteMask;
 }
}

/*******************************************************************************
* Function Name  : ENET_PhyBusyConfig		         		
* Description    : Config the status of the PHY device to Busy.
* Input          : None.
* Output         : None.	
* Return         : None.						
*******************************************************************************/
void ENET_PhyBusyConfig(FunctionalState NewState)
{
 if (NewState==ENABLE)
  ENET->MIIR|=0x01;
 else
   ENET->MIIR&=~0x01;
}

/*******************************************************************************
* Function Name  : ENET_ReadMIIData		         		
* Description    : Return the Data in the MIIDR after a read operation
* Input          : None.
* Output         : 2Bytes data read.	
* Return         : None.						
*******************************************************************************/
u16 ENET_MII_Read(u32 PhyBaseAddr , u32 PhyReg)
{
  u32 rValue;
  ENET_PhyBusyConfig(ENABLE);
  ENET_MII_WaitNotbusy();
  ENET->MIIR|=PhyBaseAddr;
  ENET->MIIR|=PhyReg<<6;
  ENET_PhyOperationConfig(READ);
  ENET_MII_WaitNotbusy();
  ENET_PhyBusyConfig(DISABLE);
  rValue=(ENET->MIIDR)& 0xFFFF;
  ENET_MII_WaitNotbusy();
  ENET_PhyBusyConfig(ENABLE);
  return rValue;
}

/*******************************************************************************
* Function Name  : ENET_WriteMIIData		         		
* Description    : Write data into the MII data register to perform a write
                   operation
* Input          : hData : Data to be written in the MII Data register.
* Output         : None.	
* Return         : None.						
*******************************************************************************/
void ENET_MII_Write(u32 PhyBaseAddr, u32 PhyReg, u16 Data)
{

  ENET_MII_WaitNotbusy();
  ENET_PhyBusyConfig(DISABLE);
   ENET_PhyBusyConfig(ENABLE);
  ENET_PhyOperationConfig(WRITE);
  ENET->MIIDR|=Data;
  ENET_MII_WaitNotbusy();
  ENET->MIIR|=PhyBaseAddr;
  ENET->MIIR|=PhyReg<<6;

 // ENET->MIIR|=addr;
  ENET_MII_WaitNotbusy();
}


void ENET_ResetMIIRegValues(void)
{
   ENET->MIIR &=0x0;
   ENET->MIIDR &=0x0;
}

void DENET_HwReset(void)
{
 u32 counter;
  /* Force a RX and TX DMA abort by software */
  DENET_Cmd(DENET_RX);
  DENET_Cmd(DENET_TX);

  DENET_SoftResetCmd(ENABLE);
    DENET->DISER&=0x00000000;
  /* loop for some cycles if needed */
  for (counter = 0; counter < 0x100; counter++);
  /* Initialize control and status registers */
  DENET->DISER&=0x00000000;
  DENET->DISR|=0xFFFFFFFF;
  DENET_Cmd(DENET_RX);
  DENET->RxDCR&=0x00000000;
  DENET->TxDCR&=0x00000000;
  /* set generic multiCAst address in the Multicast register */
  ENET_SetMulticastAddress(0xFFFFFFFF, 0xFFFFFFFF);


}

/*******************************************************************************
* Function Name  : ENET_SetControlFramePauseTime		         		
* Description    : Set the pause time in control Flow frame.
* Input          : hPtime: Pause time value
* Output         : None.	
* Return         : None.						
*******************************************************************************/
void ENET_SetControlFramePauseTime(u16 hPtime)
{
 u32 dTmp=0x0;
 dTmp|=hPtime;
 dTmp=((dTmp<<16)|0xFFFF);
 ENET->MCFR&=dTmp;
}

/*******************************************************************************
* Function Name  : ENET_EnableControlFlow		         		
* Description    : Enables or disables the control Flow frame
* Input          : None.
* Output         : None.	
* Return         : None.						
*******************************************************************************/
void ENET_ControlFlowConfig(FunctionalState NewState)
{
 if (NewState==ENABLE)
 {
  ENET->MCFR|=FCEnableMask;
 }
  else
  {
   ENET->MCFR&=FCDisableMask;
  }
}

/*******************************************************************************
* Function Name  : ENET_FlowControlBusyConfig		         		
* Description    : Config Flow control status to busy.
* Input          : None.
* Output         : None.	
* Return         : None.						
*******************************************************************************/
void ENET_FlowControlBusyConfig(void)
{
 ENET->MCFR|=0x01;
}

/*******************************************************************************
* Function Name  : ENET_GetVLANIdentifier		         		
* Description    : Get the VLAN identifier (24 bits)
* Input          : None.
* Output         : None.	
* Return         : 4 byte VLAN Identifier,the 2 upper bytes is VLANIdentifier1
                   and the lower ones is the VLANIdentifier2.								
*******************************************************************************/
u32 ENET_GetVLANIdentifier(void)
{
 u32 wResult=0x0;
 wResult|=(ENET->VL1R) & VLID_Mask;
 wResult|=(ENET->VL2R) & VLID_Mask;//>>12;
 wResult=wResult>>12;
 return wResult;
}

/*******************************************************************************
* Function Name  : ENET_GetVLANTag		         		
* Description    : GEt the VLAN Tag (32 bits)
* Input          : None.
* Output         : None.	
* Return         : 4 byte VLAN Tag, the 2 upper bytes is VLAN Tag1 and the lower
                   ones is the VLAN Tag2.					
*******************************************************************************/
u32 ENET_GetVLANTag(void)
{
 u32 wResult=0x0;
 wResult =(ENET->VL1R) & VLTAG_Mask;
 wResult =(ENET->VL2R) & VLTAG_Mask;
 wResult =wResult>>16;
 return wResult;
}

/*******************************************************************************
* Function Name  : ENET_GetVLANTag		         		
* Description    : Get the number of collision in the current transmisssion
* Input          : None.
* Output         : None.	
* Return         : Collision Counter					
*******************************************************************************/
u8 ENET_GetCollisionCounter(void)
{
 return (ENET->MTSR) & CC_Mask; /* */
}

/*******************************************************************************
* Function Name  : ENET_GetByteCounter		         		
* Description    : Get the number of byte transmitted in the current transmission
* Input          : None.
* Output         : None.	
* Return         : Transmitted byte counter. 					
*******************************************************************************/
u32 ENET_GetByteCounter(void)
{
 return (ENET->MTSR) & BC_Mask;
}

/*******************************************************************************
* Function Name  : ENET_GetByteCounter		         		
* Description    : Configure the clock relationship Hclk - Pclk
* Input          : bClockFactor : this paramater can be :
                   - ENET_Simple : HClk=PClk
                   - ENET_Double : HClk= 2*PClk
* Output         : None.	
* Return         : None. 					
*******************************************************************************/

void ENET_ClockConfig(u8 bClockFactor)
{
 if (bClockFactor == ENET_Simple)
  {
   DENET->CPR&=SimpleClock_Mask;   /*FFFFFFF3*/
  }
 if (bClockFactor == ENET_Double)
  {
   DENET->CPR |= DoubleClock_Mask;
  }
}

/*******************************************************************************
* Function Name  : ENET_GetFlagStatus		         		
* Description    : Checks whether the ENET flag is set or not.
* Input          : ENET_Flag: specifies the flag to check.
                   For more information please refer the ENET flags section in
                   the SW library user manual.
* Output         : None.	
* Return         : None. 					
*******************************************************************************/

FlagStatus ENET_GetFlagStatus(u16 ENET_FLAG)
{
 u32 wRegAddr=0x0;
 u32 wFlagIndex=0x0;
 wRegAddr = ENET_FLAG & 0xFF00;
 wFlagIndex = 0x1<<(ENET_FLAG & 0xFF);
  if (wRegAddr == 0x2800 /*MTS_Addr*/)
 {
   if ((ENET->MTSR) & wFlagIndex != RESET)
    {
    	return SET;
    }
    else
     {
      return RESET;
     }
  }
  if (wRegAddr == 0x2C00 /*MRS_Addr*/)
  {
   if ((ENET->MRSR)& wFlagIndex != RESET)
    {
    	return SET;
    }
    else
     {
      return RESET;
     }
  }
return RESET;
}










